# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from metronome import Metronome, AsyncMetronome
from tests.utils import assert_matches_type
from metronome._utils import parse_datetime
from metronome.types.v2 import (
    ContractEditResponse,
    ContractListResponse,
    ContractRetrieveResponse,
    ContractEditCommitResponse,
    ContractEditCreditResponse,
    ContractGetEditHistoryResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestContracts:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_retrieve(self, client: Metronome) -> None:
        contract = client.v2.contracts.retrieve(
            contract_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
            customer_id="13117714-3f05-48e5-a6e9-a66093f13b4d",
        )
        assert_matches_type(ContractRetrieveResponse, contract, path=["response"])

    @parametrize
    def test_method_retrieve_with_all_params(self, client: Metronome) -> None:
        contract = client.v2.contracts.retrieve(
            contract_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
            customer_id="13117714-3f05-48e5-a6e9-a66093f13b4d",
            as_of_date=parse_datetime("2019-12-27T18:11:19.117Z"),
            include_balance=True,
            include_ledgers=True,
        )
        assert_matches_type(ContractRetrieveResponse, contract, path=["response"])

    @parametrize
    def test_raw_response_retrieve(self, client: Metronome) -> None:
        response = client.v2.contracts.with_raw_response.retrieve(
            contract_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
            customer_id="13117714-3f05-48e5-a6e9-a66093f13b4d",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        contract = response.parse()
        assert_matches_type(ContractRetrieveResponse, contract, path=["response"])

    @parametrize
    def test_streaming_response_retrieve(self, client: Metronome) -> None:
        with client.v2.contracts.with_streaming_response.retrieve(
            contract_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
            customer_id="13117714-3f05-48e5-a6e9-a66093f13b4d",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            contract = response.parse()
            assert_matches_type(ContractRetrieveResponse, contract, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_list(self, client: Metronome) -> None:
        contract = client.v2.contracts.list(
            customer_id="13117714-3f05-48e5-a6e9-a66093f13b4d",
        )
        assert_matches_type(ContractListResponse, contract, path=["response"])

    @parametrize
    def test_method_list_with_all_params(self, client: Metronome) -> None:
        contract = client.v2.contracts.list(
            customer_id="13117714-3f05-48e5-a6e9-a66093f13b4d",
            covering_date=parse_datetime("2019-12-27T18:11:19.117Z"),
            include_archived=True,
            include_balance=True,
            include_ledgers=True,
            starting_at=parse_datetime("2019-12-27T18:11:19.117Z"),
        )
        assert_matches_type(ContractListResponse, contract, path=["response"])

    @parametrize
    def test_raw_response_list(self, client: Metronome) -> None:
        response = client.v2.contracts.with_raw_response.list(
            customer_id="13117714-3f05-48e5-a6e9-a66093f13b4d",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        contract = response.parse()
        assert_matches_type(ContractListResponse, contract, path=["response"])

    @parametrize
    def test_streaming_response_list(self, client: Metronome) -> None:
        with client.v2.contracts.with_streaming_response.list(
            customer_id="13117714-3f05-48e5-a6e9-a66093f13b4d",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            contract = response.parse()
            assert_matches_type(ContractListResponse, contract, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_edit(self, client: Metronome) -> None:
        contract = client.v2.contracts.edit(
            contract_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
            customer_id="13117714-3f05-48e5-a6e9-a66093f13b4d",
        )
        assert_matches_type(ContractEditResponse, contract, path=["response"])

    @parametrize
    def test_method_edit_with_all_params(self, client: Metronome) -> None:
        contract = client.v2.contracts.edit(
            contract_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
            customer_id="13117714-3f05-48e5-a6e9-a66093f13b4d",
            add_billing_provider_configuration_update={
                "billing_provider_configuration": {
                    "billing_provider": "aws_marketplace",
                    "billing_provider_configuration_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                    "delivery_method": "direct_to_billing_provider",
                },
                "schedule": {"effective_at": "START_OF_CURRENT_PERIOD"},
            },
            add_commits=[
                {
                    "product_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                    "type": "PREPAID",
                    "access_schedule": {
                        "schedule_items": [
                            {
                                "amount": 0,
                                "ending_before": parse_datetime("2019-12-27T18:11:19.117Z"),
                                "starting_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                            }
                        ],
                        "credit_type_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                    },
                    "amount": 0,
                    "applicable_product_ids": ["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
                    "applicable_product_tags": ["string"],
                    "custom_fields": {"foo": "string"},
                    "description": "description",
                    "hierarchy_configuration": {"child_access": {"type": "ALL"}},
                    "invoice_schedule": {
                        "credit_type_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                        "do_not_invoice": True,
                        "recurring_schedule": {
                            "amount_distribution": "DIVIDED",
                            "ending_before": parse_datetime("2019-12-27T18:11:19.117Z"),
                            "frequency": "MONTHLY",
                            "starting_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                            "amount": 0,
                            "quantity": 0,
                            "unit_price": 0,
                        },
                        "schedule_items": [
                            {
                                "timestamp": parse_datetime("2019-12-27T18:11:19.117Z"),
                                "amount": 0,
                                "quantity": 0,
                                "unit_price": 0,
                            }
                        ],
                    },
                    "name": "x",
                    "netsuite_sales_order_id": "netsuite_sales_order_id",
                    "payment_gate_config": {
                        "payment_gate_type": "NONE",
                        "precalculated_tax_config": {
                            "tax_amount": 0,
                            "tax_name": "tax_name",
                        },
                        "stripe_config": {
                            "payment_type": "INVOICE",
                            "invoice_metadata": {"foo": "string"},
                            "on_session_payment": True,
                        },
                        "tax_type": "NONE",
                    },
                    "priority": 0,
                    "rate_type": "COMMIT_RATE",
                    "rollover_fraction": 0,
                    "specifiers": [
                        {
                            "presentation_group_values": {"foo": "string"},
                            "pricing_group_values": {"foo": "string"},
                            "product_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                            "product_tags": ["string"],
                        }
                    ],
                    "temporary_id": "temporary_id",
                }
            ],
            add_credits=[
                {
                    "access_schedule": {
                        "schedule_items": [
                            {
                                "amount": 0,
                                "ending_before": parse_datetime("2019-12-27T18:11:19.117Z"),
                                "starting_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                            }
                        ],
                        "credit_type_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                    },
                    "product_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                    "applicable_product_ids": ["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
                    "applicable_product_tags": ["string"],
                    "custom_fields": {"foo": "string"},
                    "description": "description",
                    "hierarchy_configuration": {"child_access": {"type": "ALL"}},
                    "name": "x",
                    "netsuite_sales_order_id": "netsuite_sales_order_id",
                    "priority": 0,
                    "rate_type": "COMMIT_RATE",
                    "specifiers": [
                        {
                            "presentation_group_values": {"foo": "string"},
                            "pricing_group_values": {"foo": "string"},
                            "product_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                            "product_tags": ["string"],
                        }
                    ],
                }
            ],
            add_discounts=[
                {
                    "product_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                    "schedule": {
                        "credit_type_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                        "do_not_invoice": True,
                        "recurring_schedule": {
                            "amount_distribution": "DIVIDED",
                            "ending_before": parse_datetime("2019-12-27T18:11:19.117Z"),
                            "frequency": "MONTHLY",
                            "starting_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                            "amount": 0,
                            "quantity": 0,
                            "unit_price": 0,
                        },
                        "schedule_items": [
                            {
                                "timestamp": parse_datetime("2019-12-27T18:11:19.117Z"),
                                "amount": 0,
                                "quantity": 0,
                                "unit_price": 0,
                            }
                        ],
                    },
                    "custom_fields": {"foo": "string"},
                    "name": "x",
                    "netsuite_sales_order_id": "netsuite_sales_order_id",
                }
            ],
            add_overrides=[
                {
                    "starting_at": parse_datetime("2024-11-02T00:00:00Z"),
                    "applicable_product_tags": ["string"],
                    "ending_before": parse_datetime("2019-12-27T18:11:19.117Z"),
                    "entitled": True,
                    "is_commit_specific": True,
                    "multiplier": 2,
                    "override_specifiers": [
                        {
                            "billing_frequency": "MONTHLY",
                            "commit_ids": ["string"],
                            "presentation_group_values": {"foo": "string"},
                            "pricing_group_values": {"foo": "string"},
                            "product_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                            "product_tags": ["string"],
                            "recurring_commit_ids": ["string"],
                            "recurring_credit_ids": ["string"],
                        }
                    ],
                    "overwrite_rate": {
                        "rate_type": "FLAT",
                        "credit_type_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                        "custom_rate": {"foo": "bar"},
                        "is_prorated": True,
                        "price": 0,
                        "quantity": 0,
                        "tiers": [
                            {
                                "price": 0,
                                "size": 0,
                            }
                        ],
                    },
                    "priority": 100,
                    "product_id": "d4fc086c-d8e5-4091-a235-fbba5da4ec14",
                    "target": "COMMIT_RATE",
                    "tiers": [
                        {
                            "multiplier": 0,
                            "size": 0,
                        }
                    ],
                    "type": "MULTIPLIER",
                }
            ],
            add_prepaid_balance_threshold_configuration={
                "commit": {
                    "description": "description",
                    "name": "name",
                    "product_id": "product_id",
                    "applicable_product_ids": ["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
                    "applicable_product_tags": ["string"],
                    "specifiers": [
                        {
                            "presentation_group_values": {"foo": "string"},
                            "pricing_group_values": {"foo": "string"},
                            "product_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                            "product_tags": ["string"],
                        }
                    ],
                },
                "is_enabled": True,
                "payment_gate_config": {
                    "payment_gate_type": "NONE",
                    "precalculated_tax_config": {
                        "tax_amount": 0,
                        "tax_name": "tax_name",
                    },
                    "stripe_config": {
                        "payment_type": "INVOICE",
                        "invoice_metadata": {"foo": "string"},
                    },
                    "tax_type": "NONE",
                },
                "recharge_to_amount": 0,
                "threshold_amount": 0,
                "custom_credit_type_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            },
            add_professional_services=[
                {
                    "max_amount": 0,
                    "product_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                    "quantity": 0,
                    "unit_price": 0,
                    "custom_fields": {"foo": "string"},
                    "description": "description",
                    "netsuite_sales_order_id": "netsuite_sales_order_id",
                }
            ],
            add_recurring_commits=[
                {
                    "access_amount": {
                        "credit_type_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                        "unit_price": 0,
                        "quantity": 0,
                    },
                    "commit_duration": {
                        "value": 0,
                        "unit": "PERIODS",
                    },
                    "priority": 0,
                    "product_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                    "starting_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                    "applicable_product_ids": ["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
                    "applicable_product_tags": ["string"],
                    "description": "description",
                    "ending_before": parse_datetime("2019-12-27T18:11:19.117Z"),
                    "hierarchy_configuration": {"child_access": {"type": "ALL"}},
                    "invoice_amount": {
                        "credit_type_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                        "quantity": 0,
                        "unit_price": 0,
                    },
                    "name": "x",
                    "netsuite_sales_order_id": "netsuite_sales_order_id",
                    "proration": "NONE",
                    "rate_type": "COMMIT_RATE",
                    "recurrence_frequency": "MONTHLY",
                    "rollover_fraction": 0,
                    "specifiers": [
                        {
                            "presentation_group_values": {"foo": "string"},
                            "pricing_group_values": {"foo": "string"},
                            "product_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                            "product_tags": ["string"],
                        }
                    ],
                    "subscription_config": {
                        "apply_seat_increase_config": {"is_prorated": True},
                        "subscription_id": "subscription_id",
                        "allocation": "POOLED",
                    },
                    "temporary_id": "temporary_id",
                }
            ],
            add_recurring_credits=[
                {
                    "access_amount": {
                        "credit_type_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                        "unit_price": 0,
                        "quantity": 0,
                    },
                    "commit_duration": {
                        "value": 0,
                        "unit": "PERIODS",
                    },
                    "priority": 0,
                    "product_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                    "starting_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                    "applicable_product_ids": ["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
                    "applicable_product_tags": ["string"],
                    "description": "description",
                    "ending_before": parse_datetime("2019-12-27T18:11:19.117Z"),
                    "hierarchy_configuration": {"child_access": {"type": "ALL"}},
                    "name": "x",
                    "netsuite_sales_order_id": "netsuite_sales_order_id",
                    "proration": "NONE",
                    "rate_type": "COMMIT_RATE",
                    "recurrence_frequency": "MONTHLY",
                    "rollover_fraction": 0,
                    "specifiers": [
                        {
                            "presentation_group_values": {"foo": "string"},
                            "pricing_group_values": {"foo": "string"},
                            "product_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                            "product_tags": ["string"],
                        }
                    ],
                    "subscription_config": {
                        "apply_seat_increase_config": {"is_prorated": True},
                        "subscription_id": "subscription_id",
                        "allocation": "POOLED",
                    },
                    "temporary_id": "temporary_id",
                }
            ],
            add_reseller_royalties=[
                {
                    "reseller_type": "AWS",
                    "applicable_product_ids": ["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
                    "applicable_product_tags": ["string"],
                    "aws_options": {
                        "aws_account_number": "aws_account_number",
                        "aws_offer_id": "aws_offer_id",
                        "aws_payer_reference_id": "aws_payer_reference_id",
                    },
                    "ending_before": parse_datetime("2019-12-27T18:11:19.117Z"),
                    "fraction": 0,
                    "gcp_options": {
                        "gcp_account_id": "gcp_account_id",
                        "gcp_offer_id": "gcp_offer_id",
                    },
                    "netsuite_reseller_id": "netsuite_reseller_id",
                    "reseller_contract_value": 0,
                    "starting_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                }
            ],
            add_scheduled_charges=[
                {
                    "product_id": "2e30f074-d04c-412e-a134-851ebfa5ceb2",
                    "schedule": {
                        "credit_type_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                        "do_not_invoice": True,
                        "recurring_schedule": {
                            "amount_distribution": "DIVIDED",
                            "ending_before": parse_datetime("2019-12-27T18:11:19.117Z"),
                            "frequency": "MONTHLY",
                            "starting_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                            "amount": 0,
                            "quantity": 0,
                            "unit_price": 0,
                        },
                        "schedule_items": [
                            {
                                "timestamp": parse_datetime("2020-02-15T00:00:00.000Z"),
                                "amount": 0,
                                "quantity": 1,
                                "unit_price": 1000000,
                            }
                        ],
                    },
                    "custom_fields": {"foo": "string"},
                    "name": "x",
                    "netsuite_sales_order_id": "netsuite_sales_order_id",
                }
            ],
            add_spend_threshold_configuration={
                "commit": {
                    "description": "description",
                    "name": "name",
                    "product_id": "product_id",
                },
                "is_enabled": True,
                "payment_gate_config": {
                    "payment_gate_type": "NONE",
                    "precalculated_tax_config": {
                        "tax_amount": 0,
                        "tax_name": "tax_name",
                    },
                    "stripe_config": {
                        "payment_type": "INVOICE",
                        "invoice_metadata": {"foo": "string"},
                    },
                    "tax_type": "NONE",
                },
                "threshold_amount": 0,
            },
            add_subscriptions=[
                {
                    "collection_schedule": "ADVANCE",
                    "proration": {
                        "invoice_behavior": "BILL_IMMEDIATELY",
                        "is_prorated": True,
                    },
                    "subscription_rate": {
                        "billing_frequency": "MONTHLY",
                        "product_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                    },
                    "custom_fields": {"foo": "string"},
                    "description": "description",
                    "ending_before": parse_datetime("2019-12-27T18:11:19.117Z"),
                    "initial_quantity": 0,
                    "name": "name",
                    "quantity_management_mode": "SEAT_BASED",
                    "seat_config": {
                        "initial_seat_ids": ["string"],
                        "seat_group_key": "seat_group_key",
                        "initial_unassigned_seats": 0,
                    },
                    "starting_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                    "temporary_id": "temporary_id",
                }
            ],
            allow_contract_ending_before_finalized_invoice=True,
            archive_commits=[{"id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"}],
            archive_credits=[{"id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"}],
            archive_scheduled_charges=[{"id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"}],
            remove_overrides=[{"id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"}],
            uniqueness_key="x",
            update_commits=[
                {
                    "commit_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                    "access_schedule": {
                        "add_schedule_items": [
                            {
                                "amount": 0,
                                "ending_before": parse_datetime("2019-12-27T18:11:19.117Z"),
                                "starting_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                            }
                        ],
                        "remove_schedule_items": [{"id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"}],
                        "update_schedule_items": [
                            {
                                "id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                                "amount": 0,
                                "ending_before": parse_datetime("2019-12-27T18:11:19.117Z"),
                                "starting_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                            }
                        ],
                    },
                    "applicable_product_ids": ["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
                    "applicable_product_tags": ["string"],
                    "description": "description",
                    "hierarchy_configuration": {"child_access": {"type": "ALL"}},
                    "invoice_schedule": {
                        "add_schedule_items": [
                            {
                                "timestamp": parse_datetime("2019-12-27T18:11:19.117Z"),
                                "amount": 0,
                                "quantity": 0,
                                "unit_price": 0,
                            }
                        ],
                        "remove_schedule_items": [{"id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"}],
                        "update_schedule_items": [
                            {
                                "id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                                "amount": 0,
                                "quantity": 0,
                                "timestamp": parse_datetime("2019-12-27T18:11:19.117Z"),
                                "unit_price": 0,
                            }
                        ],
                    },
                    "name": "name",
                    "netsuite_sales_order_id": "netsuite_sales_order_id",
                    "priority": 0,
                    "product_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                    "rate_type": "LIST_RATE",
                    "rollover_fraction": 0,
                }
            ],
            update_contract_end_date=parse_datetime("2019-12-27T18:11:19.117Z"),
            update_contract_name="update_contract_name",
            update_credits=[
                {
                    "credit_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                    "access_schedule": {
                        "add_schedule_items": [
                            {
                                "amount": 0,
                                "ending_before": parse_datetime("2019-12-27T18:11:19.117Z"),
                                "starting_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                            }
                        ],
                        "remove_schedule_items": [{"id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"}],
                        "update_schedule_items": [
                            {
                                "id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                                "amount": 0,
                                "ending_before": parse_datetime("2019-12-27T18:11:19.117Z"),
                                "starting_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                            }
                        ],
                    },
                    "applicable_product_ids": ["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
                    "applicable_product_tags": ["string"],
                    "description": "description",
                    "hierarchy_configuration": {"child_access": {"type": "ALL"}},
                    "name": "name",
                    "netsuite_sales_order_id": "netsuite_sales_order_id",
                    "priority": 0,
                    "product_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                    "rate_type": "LIST_RATE",
                }
            ],
            update_prepaid_balance_threshold_configuration={
                "commit": {
                    "description": "description",
                    "name": "name",
                    "product_id": "product_id",
                    "applicable_product_ids": ["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
                    "applicable_product_tags": ["string"],
                    "specifiers": [
                        {
                            "presentation_group_values": {"foo": "string"},
                            "pricing_group_values": {"foo": "string"},
                            "product_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                            "product_tags": ["string"],
                        }
                    ],
                },
                "custom_credit_type_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                "is_enabled": True,
                "payment_gate_config": {
                    "payment_gate_type": "NONE",
                    "precalculated_tax_config": {
                        "tax_amount": 0,
                        "tax_name": "tax_name",
                    },
                    "stripe_config": {
                        "payment_type": "INVOICE",
                        "invoice_metadata": {"foo": "string"},
                    },
                    "tax_type": "NONE",
                },
                "recharge_to_amount": 0,
                "threshold_amount": 0,
            },
            update_recurring_commits=[
                {
                    "recurring_commit_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                    "access_amount": {
                        "quantity": 0,
                        "unit_price": 0,
                    },
                    "ending_before": parse_datetime("2019-12-27T18:11:19.117Z"),
                    "invoice_amount": {
                        "quantity": 0,
                        "unit_price": 0,
                    },
                    "rate_type": "LIST_RATE",
                }
            ],
            update_recurring_credits=[
                {
                    "recurring_credit_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                    "access_amount": {
                        "quantity": 0,
                        "unit_price": 0,
                    },
                    "ending_before": parse_datetime("2019-12-27T18:11:19.117Z"),
                    "rate_type": "LIST_RATE",
                }
            ],
            update_scheduled_charges=[
                {
                    "scheduled_charge_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                    "invoice_schedule": {
                        "add_schedule_items": [
                            {
                                "timestamp": parse_datetime("2019-12-27T18:11:19.117Z"),
                                "amount": 0,
                                "quantity": 0,
                                "unit_price": 0,
                            }
                        ],
                        "remove_schedule_items": [{"id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"}],
                        "update_schedule_items": [
                            {
                                "id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                                "amount": 0,
                                "quantity": 0,
                                "timestamp": parse_datetime("2019-12-27T18:11:19.117Z"),
                                "unit_price": 0,
                            }
                        ],
                    },
                    "netsuite_sales_order_id": "netsuite_sales_order_id",
                }
            ],
            update_spend_threshold_configuration={
                "commit": {
                    "description": "description",
                    "name": "name",
                    "product_id": "product_id",
                },
                "is_enabled": True,
                "payment_gate_config": {
                    "payment_gate_type": "NONE",
                    "precalculated_tax_config": {
                        "tax_amount": 0,
                        "tax_name": "tax_name",
                    },
                    "stripe_config": {
                        "payment_type": "INVOICE",
                        "invoice_metadata": {"foo": "string"},
                    },
                    "tax_type": "NONE",
                },
                "threshold_amount": 0,
            },
            update_subscriptions=[
                {
                    "subscription_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                    "ending_before": parse_datetime("2019-12-27T18:11:19.117Z"),
                    "quantity_management_mode_update": {
                        "quantity_management_mode": "SEAT_BASED",
                        "seat_config": {"seat_group_key": "seat_group_key"},
                    },
                    "quantity_updates": [
                        {
                            "starting_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                            "quantity": 0,
                            "quantity_delta": 0,
                        }
                    ],
                    "seat_updates": {
                        "add_seat_ids": [
                            {
                                "seat_ids": ["string"],
                                "starting_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                            }
                        ],
                        "add_unassigned_seats": [
                            {
                                "quantity": 1,
                                "starting_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                            }
                        ],
                        "remove_seat_ids": [
                            {
                                "seat_ids": ["string"],
                                "starting_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                            }
                        ],
                        "remove_unassigned_seats": [
                            {
                                "quantity": 1,
                                "starting_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                            }
                        ],
                    },
                }
            ],
        )
        assert_matches_type(ContractEditResponse, contract, path=["response"])

    @parametrize
    def test_raw_response_edit(self, client: Metronome) -> None:
        response = client.v2.contracts.with_raw_response.edit(
            contract_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
            customer_id="13117714-3f05-48e5-a6e9-a66093f13b4d",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        contract = response.parse()
        assert_matches_type(ContractEditResponse, contract, path=["response"])

    @parametrize
    def test_streaming_response_edit(self, client: Metronome) -> None:
        with client.v2.contracts.with_streaming_response.edit(
            contract_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
            customer_id="13117714-3f05-48e5-a6e9-a66093f13b4d",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            contract = response.parse()
            assert_matches_type(ContractEditResponse, contract, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_edit_commit(self, client: Metronome) -> None:
        contract = client.v2.contracts.edit_commit(
            commit_id="5e7e82cf-ccb7-428c-a96f-a8e4f67af822",
            customer_id="4c91c473-fc12-445a-9c38-40421d47023f",
        )
        assert_matches_type(ContractEditCommitResponse, contract, path=["response"])

    @parametrize
    def test_method_edit_commit_with_all_params(self, client: Metronome) -> None:
        contract = client.v2.contracts.edit_commit(
            commit_id="5e7e82cf-ccb7-428c-a96f-a8e4f67af822",
            customer_id="4c91c473-fc12-445a-9c38-40421d47023f",
            access_schedule={
                "add_schedule_items": [
                    {
                        "amount": 0,
                        "ending_before": parse_datetime("2019-12-27T18:11:19.117Z"),
                        "starting_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                    }
                ],
                "remove_schedule_items": [{"id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"}],
                "update_schedule_items": [
                    {
                        "id": "d5edbd32-c744-48cb-9475-a9bca0e6fa39",
                        "amount": 0,
                        "ending_before": parse_datetime("2025-03-12T00:00:00Z"),
                        "starting_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                    }
                ],
            },
            applicable_product_ids=["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
            applicable_product_tags=["string"],
            description="description",
            hierarchy_configuration={"child_access": {"type": "ALL"}},
            invoice_contract_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            invoice_schedule={
                "add_schedule_items": [
                    {
                        "timestamp": parse_datetime("2019-12-27T18:11:19.117Z"),
                        "amount": 0,
                        "quantity": 0,
                        "unit_price": 0,
                    }
                ],
                "remove_schedule_items": [{"id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"}],
                "update_schedule_items": [
                    {
                        "id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                        "amount": 0,
                        "quantity": 0,
                        "timestamp": parse_datetime("2019-12-27T18:11:19.117Z"),
                        "unit_price": 0,
                    }
                ],
            },
            name="name",
            priority=0,
            product_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            rate_type="LIST_RATE",
            specifiers=[
                {
                    "presentation_group_values": {"foo": "string"},
                    "pricing_group_values": {"foo": "string"},
                    "product_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                    "product_tags": ["string"],
                }
            ],
        )
        assert_matches_type(ContractEditCommitResponse, contract, path=["response"])

    @parametrize
    def test_raw_response_edit_commit(self, client: Metronome) -> None:
        response = client.v2.contracts.with_raw_response.edit_commit(
            commit_id="5e7e82cf-ccb7-428c-a96f-a8e4f67af822",
            customer_id="4c91c473-fc12-445a-9c38-40421d47023f",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        contract = response.parse()
        assert_matches_type(ContractEditCommitResponse, contract, path=["response"])

    @parametrize
    def test_streaming_response_edit_commit(self, client: Metronome) -> None:
        with client.v2.contracts.with_streaming_response.edit_commit(
            commit_id="5e7e82cf-ccb7-428c-a96f-a8e4f67af822",
            customer_id="4c91c473-fc12-445a-9c38-40421d47023f",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            contract = response.parse()
            assert_matches_type(ContractEditCommitResponse, contract, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_edit_credit(self, client: Metronome) -> None:
        contract = client.v2.contracts.edit_credit(
            credit_id="5e7e82cf-ccb7-428c-a96f-a8e4f67af822",
            customer_id="4c91c473-fc12-445a-9c38-40421d47023f",
        )
        assert_matches_type(ContractEditCreditResponse, contract, path=["response"])

    @parametrize
    def test_method_edit_credit_with_all_params(self, client: Metronome) -> None:
        contract = client.v2.contracts.edit_credit(
            credit_id="5e7e82cf-ccb7-428c-a96f-a8e4f67af822",
            customer_id="4c91c473-fc12-445a-9c38-40421d47023f",
            access_schedule={
                "add_schedule_items": [
                    {
                        "amount": 0,
                        "ending_before": parse_datetime("2019-12-27T18:11:19.117Z"),
                        "starting_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                    }
                ],
                "remove_schedule_items": [{"id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"}],
                "update_schedule_items": [
                    {
                        "id": "d5edbd32-c744-48cb-9475-a9bca0e6fa39",
                        "amount": 0,
                        "ending_before": parse_datetime("2025-03-12T00:00:00Z"),
                        "starting_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                    }
                ],
            },
            applicable_product_ids=["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
            applicable_product_tags=["string"],
            description="description",
            hierarchy_configuration={"child_access": {"type": "ALL"}},
            name="name",
            priority=0,
            product_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            rate_type="LIST_RATE",
            specifiers=[
                {
                    "presentation_group_values": {"foo": "string"},
                    "pricing_group_values": {"foo": "string"},
                    "product_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                    "product_tags": ["string"],
                }
            ],
        )
        assert_matches_type(ContractEditCreditResponse, contract, path=["response"])

    @parametrize
    def test_raw_response_edit_credit(self, client: Metronome) -> None:
        response = client.v2.contracts.with_raw_response.edit_credit(
            credit_id="5e7e82cf-ccb7-428c-a96f-a8e4f67af822",
            customer_id="4c91c473-fc12-445a-9c38-40421d47023f",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        contract = response.parse()
        assert_matches_type(ContractEditCreditResponse, contract, path=["response"])

    @parametrize
    def test_streaming_response_edit_credit(self, client: Metronome) -> None:
        with client.v2.contracts.with_streaming_response.edit_credit(
            credit_id="5e7e82cf-ccb7-428c-a96f-a8e4f67af822",
            customer_id="4c91c473-fc12-445a-9c38-40421d47023f",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            contract = response.parse()
            assert_matches_type(ContractEditCreditResponse, contract, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_get_edit_history(self, client: Metronome) -> None:
        contract = client.v2.contracts.get_edit_history(
            contract_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
            customer_id="13117714-3f05-48e5-a6e9-a66093f13b4d",
        )
        assert_matches_type(ContractGetEditHistoryResponse, contract, path=["response"])

    @parametrize
    def test_raw_response_get_edit_history(self, client: Metronome) -> None:
        response = client.v2.contracts.with_raw_response.get_edit_history(
            contract_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
            customer_id="13117714-3f05-48e5-a6e9-a66093f13b4d",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        contract = response.parse()
        assert_matches_type(ContractGetEditHistoryResponse, contract, path=["response"])

    @parametrize
    def test_streaming_response_get_edit_history(self, client: Metronome) -> None:
        with client.v2.contracts.with_streaming_response.get_edit_history(
            contract_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
            customer_id="13117714-3f05-48e5-a6e9-a66093f13b4d",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            contract = response.parse()
            assert_matches_type(ContractGetEditHistoryResponse, contract, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncContracts:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @parametrize
    async def test_method_retrieve(self, async_client: AsyncMetronome) -> None:
        contract = await async_client.v2.contracts.retrieve(
            contract_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
            customer_id="13117714-3f05-48e5-a6e9-a66093f13b4d",
        )
        assert_matches_type(ContractRetrieveResponse, contract, path=["response"])

    @parametrize
    async def test_method_retrieve_with_all_params(self, async_client: AsyncMetronome) -> None:
        contract = await async_client.v2.contracts.retrieve(
            contract_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
            customer_id="13117714-3f05-48e5-a6e9-a66093f13b4d",
            as_of_date=parse_datetime("2019-12-27T18:11:19.117Z"),
            include_balance=True,
            include_ledgers=True,
        )
        assert_matches_type(ContractRetrieveResponse, contract, path=["response"])

    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncMetronome) -> None:
        response = await async_client.v2.contracts.with_raw_response.retrieve(
            contract_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
            customer_id="13117714-3f05-48e5-a6e9-a66093f13b4d",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        contract = await response.parse()
        assert_matches_type(ContractRetrieveResponse, contract, path=["response"])

    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncMetronome) -> None:
        async with async_client.v2.contracts.with_streaming_response.retrieve(
            contract_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
            customer_id="13117714-3f05-48e5-a6e9-a66093f13b4d",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            contract = await response.parse()
            assert_matches_type(ContractRetrieveResponse, contract, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_list(self, async_client: AsyncMetronome) -> None:
        contract = await async_client.v2.contracts.list(
            customer_id="13117714-3f05-48e5-a6e9-a66093f13b4d",
        )
        assert_matches_type(ContractListResponse, contract, path=["response"])

    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncMetronome) -> None:
        contract = await async_client.v2.contracts.list(
            customer_id="13117714-3f05-48e5-a6e9-a66093f13b4d",
            covering_date=parse_datetime("2019-12-27T18:11:19.117Z"),
            include_archived=True,
            include_balance=True,
            include_ledgers=True,
            starting_at=parse_datetime("2019-12-27T18:11:19.117Z"),
        )
        assert_matches_type(ContractListResponse, contract, path=["response"])

    @parametrize
    async def test_raw_response_list(self, async_client: AsyncMetronome) -> None:
        response = await async_client.v2.contracts.with_raw_response.list(
            customer_id="13117714-3f05-48e5-a6e9-a66093f13b4d",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        contract = await response.parse()
        assert_matches_type(ContractListResponse, contract, path=["response"])

    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncMetronome) -> None:
        async with async_client.v2.contracts.with_streaming_response.list(
            customer_id="13117714-3f05-48e5-a6e9-a66093f13b4d",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            contract = await response.parse()
            assert_matches_type(ContractListResponse, contract, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_edit(self, async_client: AsyncMetronome) -> None:
        contract = await async_client.v2.contracts.edit(
            contract_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
            customer_id="13117714-3f05-48e5-a6e9-a66093f13b4d",
        )
        assert_matches_type(ContractEditResponse, contract, path=["response"])

    @parametrize
    async def test_method_edit_with_all_params(self, async_client: AsyncMetronome) -> None:
        contract = await async_client.v2.contracts.edit(
            contract_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
            customer_id="13117714-3f05-48e5-a6e9-a66093f13b4d",
            add_billing_provider_configuration_update={
                "billing_provider_configuration": {
                    "billing_provider": "aws_marketplace",
                    "billing_provider_configuration_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                    "delivery_method": "direct_to_billing_provider",
                },
                "schedule": {"effective_at": "START_OF_CURRENT_PERIOD"},
            },
            add_commits=[
                {
                    "product_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                    "type": "PREPAID",
                    "access_schedule": {
                        "schedule_items": [
                            {
                                "amount": 0,
                                "ending_before": parse_datetime("2019-12-27T18:11:19.117Z"),
                                "starting_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                            }
                        ],
                        "credit_type_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                    },
                    "amount": 0,
                    "applicable_product_ids": ["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
                    "applicable_product_tags": ["string"],
                    "custom_fields": {"foo": "string"},
                    "description": "description",
                    "hierarchy_configuration": {"child_access": {"type": "ALL"}},
                    "invoice_schedule": {
                        "credit_type_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                        "do_not_invoice": True,
                        "recurring_schedule": {
                            "amount_distribution": "DIVIDED",
                            "ending_before": parse_datetime("2019-12-27T18:11:19.117Z"),
                            "frequency": "MONTHLY",
                            "starting_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                            "amount": 0,
                            "quantity": 0,
                            "unit_price": 0,
                        },
                        "schedule_items": [
                            {
                                "timestamp": parse_datetime("2019-12-27T18:11:19.117Z"),
                                "amount": 0,
                                "quantity": 0,
                                "unit_price": 0,
                            }
                        ],
                    },
                    "name": "x",
                    "netsuite_sales_order_id": "netsuite_sales_order_id",
                    "payment_gate_config": {
                        "payment_gate_type": "NONE",
                        "precalculated_tax_config": {
                            "tax_amount": 0,
                            "tax_name": "tax_name",
                        },
                        "stripe_config": {
                            "payment_type": "INVOICE",
                            "invoice_metadata": {"foo": "string"},
                            "on_session_payment": True,
                        },
                        "tax_type": "NONE",
                    },
                    "priority": 0,
                    "rate_type": "COMMIT_RATE",
                    "rollover_fraction": 0,
                    "specifiers": [
                        {
                            "presentation_group_values": {"foo": "string"},
                            "pricing_group_values": {"foo": "string"},
                            "product_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                            "product_tags": ["string"],
                        }
                    ],
                    "temporary_id": "temporary_id",
                }
            ],
            add_credits=[
                {
                    "access_schedule": {
                        "schedule_items": [
                            {
                                "amount": 0,
                                "ending_before": parse_datetime("2019-12-27T18:11:19.117Z"),
                                "starting_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                            }
                        ],
                        "credit_type_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                    },
                    "product_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                    "applicable_product_ids": ["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
                    "applicable_product_tags": ["string"],
                    "custom_fields": {"foo": "string"},
                    "description": "description",
                    "hierarchy_configuration": {"child_access": {"type": "ALL"}},
                    "name": "x",
                    "netsuite_sales_order_id": "netsuite_sales_order_id",
                    "priority": 0,
                    "rate_type": "COMMIT_RATE",
                    "specifiers": [
                        {
                            "presentation_group_values": {"foo": "string"},
                            "pricing_group_values": {"foo": "string"},
                            "product_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                            "product_tags": ["string"],
                        }
                    ],
                }
            ],
            add_discounts=[
                {
                    "product_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                    "schedule": {
                        "credit_type_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                        "do_not_invoice": True,
                        "recurring_schedule": {
                            "amount_distribution": "DIVIDED",
                            "ending_before": parse_datetime("2019-12-27T18:11:19.117Z"),
                            "frequency": "MONTHLY",
                            "starting_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                            "amount": 0,
                            "quantity": 0,
                            "unit_price": 0,
                        },
                        "schedule_items": [
                            {
                                "timestamp": parse_datetime("2019-12-27T18:11:19.117Z"),
                                "amount": 0,
                                "quantity": 0,
                                "unit_price": 0,
                            }
                        ],
                    },
                    "custom_fields": {"foo": "string"},
                    "name": "x",
                    "netsuite_sales_order_id": "netsuite_sales_order_id",
                }
            ],
            add_overrides=[
                {
                    "starting_at": parse_datetime("2024-11-02T00:00:00Z"),
                    "applicable_product_tags": ["string"],
                    "ending_before": parse_datetime("2019-12-27T18:11:19.117Z"),
                    "entitled": True,
                    "is_commit_specific": True,
                    "multiplier": 2,
                    "override_specifiers": [
                        {
                            "billing_frequency": "MONTHLY",
                            "commit_ids": ["string"],
                            "presentation_group_values": {"foo": "string"},
                            "pricing_group_values": {"foo": "string"},
                            "product_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                            "product_tags": ["string"],
                            "recurring_commit_ids": ["string"],
                            "recurring_credit_ids": ["string"],
                        }
                    ],
                    "overwrite_rate": {
                        "rate_type": "FLAT",
                        "credit_type_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                        "custom_rate": {"foo": "bar"},
                        "is_prorated": True,
                        "price": 0,
                        "quantity": 0,
                        "tiers": [
                            {
                                "price": 0,
                                "size": 0,
                            }
                        ],
                    },
                    "priority": 100,
                    "product_id": "d4fc086c-d8e5-4091-a235-fbba5da4ec14",
                    "target": "COMMIT_RATE",
                    "tiers": [
                        {
                            "multiplier": 0,
                            "size": 0,
                        }
                    ],
                    "type": "MULTIPLIER",
                }
            ],
            add_prepaid_balance_threshold_configuration={
                "commit": {
                    "description": "description",
                    "name": "name",
                    "product_id": "product_id",
                    "applicable_product_ids": ["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
                    "applicable_product_tags": ["string"],
                    "specifiers": [
                        {
                            "presentation_group_values": {"foo": "string"},
                            "pricing_group_values": {"foo": "string"},
                            "product_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                            "product_tags": ["string"],
                        }
                    ],
                },
                "is_enabled": True,
                "payment_gate_config": {
                    "payment_gate_type": "NONE",
                    "precalculated_tax_config": {
                        "tax_amount": 0,
                        "tax_name": "tax_name",
                    },
                    "stripe_config": {
                        "payment_type": "INVOICE",
                        "invoice_metadata": {"foo": "string"},
                    },
                    "tax_type": "NONE",
                },
                "recharge_to_amount": 0,
                "threshold_amount": 0,
                "custom_credit_type_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            },
            add_professional_services=[
                {
                    "max_amount": 0,
                    "product_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                    "quantity": 0,
                    "unit_price": 0,
                    "custom_fields": {"foo": "string"},
                    "description": "description",
                    "netsuite_sales_order_id": "netsuite_sales_order_id",
                }
            ],
            add_recurring_commits=[
                {
                    "access_amount": {
                        "credit_type_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                        "unit_price": 0,
                        "quantity": 0,
                    },
                    "commit_duration": {
                        "value": 0,
                        "unit": "PERIODS",
                    },
                    "priority": 0,
                    "product_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                    "starting_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                    "applicable_product_ids": ["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
                    "applicable_product_tags": ["string"],
                    "description": "description",
                    "ending_before": parse_datetime("2019-12-27T18:11:19.117Z"),
                    "hierarchy_configuration": {"child_access": {"type": "ALL"}},
                    "invoice_amount": {
                        "credit_type_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                        "quantity": 0,
                        "unit_price": 0,
                    },
                    "name": "x",
                    "netsuite_sales_order_id": "netsuite_sales_order_id",
                    "proration": "NONE",
                    "rate_type": "COMMIT_RATE",
                    "recurrence_frequency": "MONTHLY",
                    "rollover_fraction": 0,
                    "specifiers": [
                        {
                            "presentation_group_values": {"foo": "string"},
                            "pricing_group_values": {"foo": "string"},
                            "product_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                            "product_tags": ["string"],
                        }
                    ],
                    "subscription_config": {
                        "apply_seat_increase_config": {"is_prorated": True},
                        "subscription_id": "subscription_id",
                        "allocation": "POOLED",
                    },
                    "temporary_id": "temporary_id",
                }
            ],
            add_recurring_credits=[
                {
                    "access_amount": {
                        "credit_type_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                        "unit_price": 0,
                        "quantity": 0,
                    },
                    "commit_duration": {
                        "value": 0,
                        "unit": "PERIODS",
                    },
                    "priority": 0,
                    "product_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                    "starting_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                    "applicable_product_ids": ["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
                    "applicable_product_tags": ["string"],
                    "description": "description",
                    "ending_before": parse_datetime("2019-12-27T18:11:19.117Z"),
                    "hierarchy_configuration": {"child_access": {"type": "ALL"}},
                    "name": "x",
                    "netsuite_sales_order_id": "netsuite_sales_order_id",
                    "proration": "NONE",
                    "rate_type": "COMMIT_RATE",
                    "recurrence_frequency": "MONTHLY",
                    "rollover_fraction": 0,
                    "specifiers": [
                        {
                            "presentation_group_values": {"foo": "string"},
                            "pricing_group_values": {"foo": "string"},
                            "product_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                            "product_tags": ["string"],
                        }
                    ],
                    "subscription_config": {
                        "apply_seat_increase_config": {"is_prorated": True},
                        "subscription_id": "subscription_id",
                        "allocation": "POOLED",
                    },
                    "temporary_id": "temporary_id",
                }
            ],
            add_reseller_royalties=[
                {
                    "reseller_type": "AWS",
                    "applicable_product_ids": ["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
                    "applicable_product_tags": ["string"],
                    "aws_options": {
                        "aws_account_number": "aws_account_number",
                        "aws_offer_id": "aws_offer_id",
                        "aws_payer_reference_id": "aws_payer_reference_id",
                    },
                    "ending_before": parse_datetime("2019-12-27T18:11:19.117Z"),
                    "fraction": 0,
                    "gcp_options": {
                        "gcp_account_id": "gcp_account_id",
                        "gcp_offer_id": "gcp_offer_id",
                    },
                    "netsuite_reseller_id": "netsuite_reseller_id",
                    "reseller_contract_value": 0,
                    "starting_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                }
            ],
            add_scheduled_charges=[
                {
                    "product_id": "2e30f074-d04c-412e-a134-851ebfa5ceb2",
                    "schedule": {
                        "credit_type_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                        "do_not_invoice": True,
                        "recurring_schedule": {
                            "amount_distribution": "DIVIDED",
                            "ending_before": parse_datetime("2019-12-27T18:11:19.117Z"),
                            "frequency": "MONTHLY",
                            "starting_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                            "amount": 0,
                            "quantity": 0,
                            "unit_price": 0,
                        },
                        "schedule_items": [
                            {
                                "timestamp": parse_datetime("2020-02-15T00:00:00.000Z"),
                                "amount": 0,
                                "quantity": 1,
                                "unit_price": 1000000,
                            }
                        ],
                    },
                    "custom_fields": {"foo": "string"},
                    "name": "x",
                    "netsuite_sales_order_id": "netsuite_sales_order_id",
                }
            ],
            add_spend_threshold_configuration={
                "commit": {
                    "description": "description",
                    "name": "name",
                    "product_id": "product_id",
                },
                "is_enabled": True,
                "payment_gate_config": {
                    "payment_gate_type": "NONE",
                    "precalculated_tax_config": {
                        "tax_amount": 0,
                        "tax_name": "tax_name",
                    },
                    "stripe_config": {
                        "payment_type": "INVOICE",
                        "invoice_metadata": {"foo": "string"},
                    },
                    "tax_type": "NONE",
                },
                "threshold_amount": 0,
            },
            add_subscriptions=[
                {
                    "collection_schedule": "ADVANCE",
                    "proration": {
                        "invoice_behavior": "BILL_IMMEDIATELY",
                        "is_prorated": True,
                    },
                    "subscription_rate": {
                        "billing_frequency": "MONTHLY",
                        "product_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                    },
                    "custom_fields": {"foo": "string"},
                    "description": "description",
                    "ending_before": parse_datetime("2019-12-27T18:11:19.117Z"),
                    "initial_quantity": 0,
                    "name": "name",
                    "quantity_management_mode": "SEAT_BASED",
                    "seat_config": {
                        "initial_seat_ids": ["string"],
                        "seat_group_key": "seat_group_key",
                        "initial_unassigned_seats": 0,
                    },
                    "starting_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                    "temporary_id": "temporary_id",
                }
            ],
            allow_contract_ending_before_finalized_invoice=True,
            archive_commits=[{"id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"}],
            archive_credits=[{"id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"}],
            archive_scheduled_charges=[{"id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"}],
            remove_overrides=[{"id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"}],
            uniqueness_key="x",
            update_commits=[
                {
                    "commit_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                    "access_schedule": {
                        "add_schedule_items": [
                            {
                                "amount": 0,
                                "ending_before": parse_datetime("2019-12-27T18:11:19.117Z"),
                                "starting_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                            }
                        ],
                        "remove_schedule_items": [{"id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"}],
                        "update_schedule_items": [
                            {
                                "id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                                "amount": 0,
                                "ending_before": parse_datetime("2019-12-27T18:11:19.117Z"),
                                "starting_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                            }
                        ],
                    },
                    "applicable_product_ids": ["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
                    "applicable_product_tags": ["string"],
                    "description": "description",
                    "hierarchy_configuration": {"child_access": {"type": "ALL"}},
                    "invoice_schedule": {
                        "add_schedule_items": [
                            {
                                "timestamp": parse_datetime("2019-12-27T18:11:19.117Z"),
                                "amount": 0,
                                "quantity": 0,
                                "unit_price": 0,
                            }
                        ],
                        "remove_schedule_items": [{"id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"}],
                        "update_schedule_items": [
                            {
                                "id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                                "amount": 0,
                                "quantity": 0,
                                "timestamp": parse_datetime("2019-12-27T18:11:19.117Z"),
                                "unit_price": 0,
                            }
                        ],
                    },
                    "name": "name",
                    "netsuite_sales_order_id": "netsuite_sales_order_id",
                    "priority": 0,
                    "product_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                    "rate_type": "LIST_RATE",
                    "rollover_fraction": 0,
                }
            ],
            update_contract_end_date=parse_datetime("2019-12-27T18:11:19.117Z"),
            update_contract_name="update_contract_name",
            update_credits=[
                {
                    "credit_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                    "access_schedule": {
                        "add_schedule_items": [
                            {
                                "amount": 0,
                                "ending_before": parse_datetime("2019-12-27T18:11:19.117Z"),
                                "starting_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                            }
                        ],
                        "remove_schedule_items": [{"id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"}],
                        "update_schedule_items": [
                            {
                                "id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                                "amount": 0,
                                "ending_before": parse_datetime("2019-12-27T18:11:19.117Z"),
                                "starting_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                            }
                        ],
                    },
                    "applicable_product_ids": ["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
                    "applicable_product_tags": ["string"],
                    "description": "description",
                    "hierarchy_configuration": {"child_access": {"type": "ALL"}},
                    "name": "name",
                    "netsuite_sales_order_id": "netsuite_sales_order_id",
                    "priority": 0,
                    "product_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                    "rate_type": "LIST_RATE",
                }
            ],
            update_prepaid_balance_threshold_configuration={
                "commit": {
                    "description": "description",
                    "name": "name",
                    "product_id": "product_id",
                    "applicable_product_ids": ["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
                    "applicable_product_tags": ["string"],
                    "specifiers": [
                        {
                            "presentation_group_values": {"foo": "string"},
                            "pricing_group_values": {"foo": "string"},
                            "product_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                            "product_tags": ["string"],
                        }
                    ],
                },
                "custom_credit_type_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                "is_enabled": True,
                "payment_gate_config": {
                    "payment_gate_type": "NONE",
                    "precalculated_tax_config": {
                        "tax_amount": 0,
                        "tax_name": "tax_name",
                    },
                    "stripe_config": {
                        "payment_type": "INVOICE",
                        "invoice_metadata": {"foo": "string"},
                    },
                    "tax_type": "NONE",
                },
                "recharge_to_amount": 0,
                "threshold_amount": 0,
            },
            update_recurring_commits=[
                {
                    "recurring_commit_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                    "access_amount": {
                        "quantity": 0,
                        "unit_price": 0,
                    },
                    "ending_before": parse_datetime("2019-12-27T18:11:19.117Z"),
                    "invoice_amount": {
                        "quantity": 0,
                        "unit_price": 0,
                    },
                    "rate_type": "LIST_RATE",
                }
            ],
            update_recurring_credits=[
                {
                    "recurring_credit_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                    "access_amount": {
                        "quantity": 0,
                        "unit_price": 0,
                    },
                    "ending_before": parse_datetime("2019-12-27T18:11:19.117Z"),
                    "rate_type": "LIST_RATE",
                }
            ],
            update_scheduled_charges=[
                {
                    "scheduled_charge_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                    "invoice_schedule": {
                        "add_schedule_items": [
                            {
                                "timestamp": parse_datetime("2019-12-27T18:11:19.117Z"),
                                "amount": 0,
                                "quantity": 0,
                                "unit_price": 0,
                            }
                        ],
                        "remove_schedule_items": [{"id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"}],
                        "update_schedule_items": [
                            {
                                "id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                                "amount": 0,
                                "quantity": 0,
                                "timestamp": parse_datetime("2019-12-27T18:11:19.117Z"),
                                "unit_price": 0,
                            }
                        ],
                    },
                    "netsuite_sales_order_id": "netsuite_sales_order_id",
                }
            ],
            update_spend_threshold_configuration={
                "commit": {
                    "description": "description",
                    "name": "name",
                    "product_id": "product_id",
                },
                "is_enabled": True,
                "payment_gate_config": {
                    "payment_gate_type": "NONE",
                    "precalculated_tax_config": {
                        "tax_amount": 0,
                        "tax_name": "tax_name",
                    },
                    "stripe_config": {
                        "payment_type": "INVOICE",
                        "invoice_metadata": {"foo": "string"},
                    },
                    "tax_type": "NONE",
                },
                "threshold_amount": 0,
            },
            update_subscriptions=[
                {
                    "subscription_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                    "ending_before": parse_datetime("2019-12-27T18:11:19.117Z"),
                    "quantity_management_mode_update": {
                        "quantity_management_mode": "SEAT_BASED",
                        "seat_config": {"seat_group_key": "seat_group_key"},
                    },
                    "quantity_updates": [
                        {
                            "starting_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                            "quantity": 0,
                            "quantity_delta": 0,
                        }
                    ],
                    "seat_updates": {
                        "add_seat_ids": [
                            {
                                "seat_ids": ["string"],
                                "starting_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                            }
                        ],
                        "add_unassigned_seats": [
                            {
                                "quantity": 1,
                                "starting_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                            }
                        ],
                        "remove_seat_ids": [
                            {
                                "seat_ids": ["string"],
                                "starting_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                            }
                        ],
                        "remove_unassigned_seats": [
                            {
                                "quantity": 1,
                                "starting_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                            }
                        ],
                    },
                }
            ],
        )
        assert_matches_type(ContractEditResponse, contract, path=["response"])

    @parametrize
    async def test_raw_response_edit(self, async_client: AsyncMetronome) -> None:
        response = await async_client.v2.contracts.with_raw_response.edit(
            contract_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
            customer_id="13117714-3f05-48e5-a6e9-a66093f13b4d",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        contract = await response.parse()
        assert_matches_type(ContractEditResponse, contract, path=["response"])

    @parametrize
    async def test_streaming_response_edit(self, async_client: AsyncMetronome) -> None:
        async with async_client.v2.contracts.with_streaming_response.edit(
            contract_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
            customer_id="13117714-3f05-48e5-a6e9-a66093f13b4d",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            contract = await response.parse()
            assert_matches_type(ContractEditResponse, contract, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_edit_commit(self, async_client: AsyncMetronome) -> None:
        contract = await async_client.v2.contracts.edit_commit(
            commit_id="5e7e82cf-ccb7-428c-a96f-a8e4f67af822",
            customer_id="4c91c473-fc12-445a-9c38-40421d47023f",
        )
        assert_matches_type(ContractEditCommitResponse, contract, path=["response"])

    @parametrize
    async def test_method_edit_commit_with_all_params(self, async_client: AsyncMetronome) -> None:
        contract = await async_client.v2.contracts.edit_commit(
            commit_id="5e7e82cf-ccb7-428c-a96f-a8e4f67af822",
            customer_id="4c91c473-fc12-445a-9c38-40421d47023f",
            access_schedule={
                "add_schedule_items": [
                    {
                        "amount": 0,
                        "ending_before": parse_datetime("2019-12-27T18:11:19.117Z"),
                        "starting_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                    }
                ],
                "remove_schedule_items": [{"id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"}],
                "update_schedule_items": [
                    {
                        "id": "d5edbd32-c744-48cb-9475-a9bca0e6fa39",
                        "amount": 0,
                        "ending_before": parse_datetime("2025-03-12T00:00:00Z"),
                        "starting_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                    }
                ],
            },
            applicable_product_ids=["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
            applicable_product_tags=["string"],
            description="description",
            hierarchy_configuration={"child_access": {"type": "ALL"}},
            invoice_contract_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            invoice_schedule={
                "add_schedule_items": [
                    {
                        "timestamp": parse_datetime("2019-12-27T18:11:19.117Z"),
                        "amount": 0,
                        "quantity": 0,
                        "unit_price": 0,
                    }
                ],
                "remove_schedule_items": [{"id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"}],
                "update_schedule_items": [
                    {
                        "id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                        "amount": 0,
                        "quantity": 0,
                        "timestamp": parse_datetime("2019-12-27T18:11:19.117Z"),
                        "unit_price": 0,
                    }
                ],
            },
            name="name",
            priority=0,
            product_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            rate_type="LIST_RATE",
            specifiers=[
                {
                    "presentation_group_values": {"foo": "string"},
                    "pricing_group_values": {"foo": "string"},
                    "product_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                    "product_tags": ["string"],
                }
            ],
        )
        assert_matches_type(ContractEditCommitResponse, contract, path=["response"])

    @parametrize
    async def test_raw_response_edit_commit(self, async_client: AsyncMetronome) -> None:
        response = await async_client.v2.contracts.with_raw_response.edit_commit(
            commit_id="5e7e82cf-ccb7-428c-a96f-a8e4f67af822",
            customer_id="4c91c473-fc12-445a-9c38-40421d47023f",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        contract = await response.parse()
        assert_matches_type(ContractEditCommitResponse, contract, path=["response"])

    @parametrize
    async def test_streaming_response_edit_commit(self, async_client: AsyncMetronome) -> None:
        async with async_client.v2.contracts.with_streaming_response.edit_commit(
            commit_id="5e7e82cf-ccb7-428c-a96f-a8e4f67af822",
            customer_id="4c91c473-fc12-445a-9c38-40421d47023f",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            contract = await response.parse()
            assert_matches_type(ContractEditCommitResponse, contract, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_edit_credit(self, async_client: AsyncMetronome) -> None:
        contract = await async_client.v2.contracts.edit_credit(
            credit_id="5e7e82cf-ccb7-428c-a96f-a8e4f67af822",
            customer_id="4c91c473-fc12-445a-9c38-40421d47023f",
        )
        assert_matches_type(ContractEditCreditResponse, contract, path=["response"])

    @parametrize
    async def test_method_edit_credit_with_all_params(self, async_client: AsyncMetronome) -> None:
        contract = await async_client.v2.contracts.edit_credit(
            credit_id="5e7e82cf-ccb7-428c-a96f-a8e4f67af822",
            customer_id="4c91c473-fc12-445a-9c38-40421d47023f",
            access_schedule={
                "add_schedule_items": [
                    {
                        "amount": 0,
                        "ending_before": parse_datetime("2019-12-27T18:11:19.117Z"),
                        "starting_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                    }
                ],
                "remove_schedule_items": [{"id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"}],
                "update_schedule_items": [
                    {
                        "id": "d5edbd32-c744-48cb-9475-a9bca0e6fa39",
                        "amount": 0,
                        "ending_before": parse_datetime("2025-03-12T00:00:00Z"),
                        "starting_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                    }
                ],
            },
            applicable_product_ids=["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
            applicable_product_tags=["string"],
            description="description",
            hierarchy_configuration={"child_access": {"type": "ALL"}},
            name="name",
            priority=0,
            product_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            rate_type="LIST_RATE",
            specifiers=[
                {
                    "presentation_group_values": {"foo": "string"},
                    "pricing_group_values": {"foo": "string"},
                    "product_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                    "product_tags": ["string"],
                }
            ],
        )
        assert_matches_type(ContractEditCreditResponse, contract, path=["response"])

    @parametrize
    async def test_raw_response_edit_credit(self, async_client: AsyncMetronome) -> None:
        response = await async_client.v2.contracts.with_raw_response.edit_credit(
            credit_id="5e7e82cf-ccb7-428c-a96f-a8e4f67af822",
            customer_id="4c91c473-fc12-445a-9c38-40421d47023f",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        contract = await response.parse()
        assert_matches_type(ContractEditCreditResponse, contract, path=["response"])

    @parametrize
    async def test_streaming_response_edit_credit(self, async_client: AsyncMetronome) -> None:
        async with async_client.v2.contracts.with_streaming_response.edit_credit(
            credit_id="5e7e82cf-ccb7-428c-a96f-a8e4f67af822",
            customer_id="4c91c473-fc12-445a-9c38-40421d47023f",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            contract = await response.parse()
            assert_matches_type(ContractEditCreditResponse, contract, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_get_edit_history(self, async_client: AsyncMetronome) -> None:
        contract = await async_client.v2.contracts.get_edit_history(
            contract_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
            customer_id="13117714-3f05-48e5-a6e9-a66093f13b4d",
        )
        assert_matches_type(ContractGetEditHistoryResponse, contract, path=["response"])

    @parametrize
    async def test_raw_response_get_edit_history(self, async_client: AsyncMetronome) -> None:
        response = await async_client.v2.contracts.with_raw_response.get_edit_history(
            contract_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
            customer_id="13117714-3f05-48e5-a6e9-a66093f13b4d",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        contract = await response.parse()
        assert_matches_type(ContractGetEditHistoryResponse, contract, path=["response"])

    @parametrize
    async def test_streaming_response_get_edit_history(self, async_client: AsyncMetronome) -> None:
        async with async_client.v2.contracts.with_streaming_response.get_edit_history(
            contract_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
            customer_id="13117714-3f05-48e5-a6e9-a66093f13b4d",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            contract = await response.parse()
            assert_matches_type(ContractGetEditHistoryResponse, contract, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .payment import Payment
from ..._models import BaseModel

__all__ = ["PaymentCancelResponse"]


class PaymentCancelResponse(BaseModel):
    data: Payment



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import datetime
from typing_extensions import Required, Annotated, TypedDict

from ...._utils import PropertyInfo

__all__ = ["CommitUpdateEndDateParams"]


class CommitUpdateEndDateParams(TypedDict, total=False):
    commit_id: Required[str]
    """ID of the commit to update. Only supports "PREPAID" commits."""

    customer_id: Required[str]
    """ID of the customer whose commit is to be updated"""

    access_ending_before: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """
    RFC 3339 timestamp indicating when access to the commit will end and it will no
    longer be possible to draw it down (exclusive). If not provided, the access will
    not be updated.
    """

    invoices_ending_before: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """
    RFC 3339 timestamp indicating when the commit will stop being invoiced
    (exclusive). If not provided, the invoice schedule will not be updated.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict
from typing_extensions import TypedDict

from ..._types import SequenceNotStr

__all__ = ["CommitSpecifierInput"]


class CommitSpecifierInput(TypedDict, total=False):
    presentation_group_values: Dict[str, str]

    pricing_group_values: Dict[str, str]

    product_id: str
    """
    If provided, the specifier will only apply to the product with the specified ID.
    """

    product_tags: SequenceNotStr[str]
    """
    If provided, the specifier will only apply to products with all the specified
    tags.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import datetime
from typing_extensions import Required, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["ContractListParams"]


class ContractListParams(TypedDict, total=False):
    customer_id: Required[str]

    covering_date: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """Optional RFC 3339 timestamp.

    Only include contracts active on the provided date. This cannot be provided if
    starting_at filter is provided.
    """

    include_archived: bool
    """Include archived contracts in the response."""

    include_balance: bool
    """Include the balance of credits and commits in the response.

    Setting this flag may cause the response to be slower.
    """

    include_ledgers: bool
    """Include commit/credit ledgers in the response.

    Setting this flag may cause the response to be slower.
    """

    starting_at: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """Optional RFC 3339 timestamp.

    Only include contracts that started on or after this date. This cannot be
    provided if covering_date filter is provided.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from metronome import Metronome, AsyncMetronome
from tests.utils import assert_matches_type
from metronome.types.v1 import DashboardGetEmbeddableURLResponse

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestDashboards:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_get_embeddable_url(self, client: Metronome) -> None:
        dashboard = client.v1.dashboards.get_embeddable_url(
            customer_id="4db51251-61de-4bfe-b9ce-495e244f3491",
            dashboard="invoices",
        )
        assert_matches_type(DashboardGetEmbeddableURLResponse, dashboard, path=["response"])

    @parametrize
    def test_method_get_embeddable_url_with_all_params(self, client: Metronome) -> None:
        dashboard = client.v1.dashboards.get_embeddable_url(
            customer_id="4db51251-61de-4bfe-b9ce-495e244f3491",
            dashboard="invoices",
            bm_group_key_overrides=[
                {
                    "group_key_name": "tenant_id",
                    "display_name": "Org ID",
                    "value_display_names": {
                        "48ecb18f358f": "bar",
                        "e358f3ce242d": "bar",
                    },
                }
            ],
            color_overrides=[
                {
                    "name": "Gray_dark",
                    "value": "#ff0000",
                }
            ],
            dashboard_options=[
                {
                    "key": "show_zero_usage_line_items",
                    "value": "false",
                },
                {
                    "key": "hide_voided_invoices",
                    "value": "true",
                },
            ],
        )
        assert_matches_type(DashboardGetEmbeddableURLResponse, dashboard, path=["response"])

    @parametrize
    def test_raw_response_get_embeddable_url(self, client: Metronome) -> None:
        response = client.v1.dashboards.with_raw_response.get_embeddable_url(
            customer_id="4db51251-61de-4bfe-b9ce-495e244f3491",
            dashboard="invoices",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        dashboard = response.parse()
        assert_matches_type(DashboardGetEmbeddableURLResponse, dashboard, path=["response"])

    @parametrize
    def test_streaming_response_get_embeddable_url(self, client: Metronome) -> None:
        with client.v1.dashboards.with_streaming_response.get_embeddable_url(
            customer_id="4db51251-61de-4bfe-b9ce-495e244f3491",
            dashboard="invoices",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            dashboard = response.parse()
            assert_matches_type(DashboardGetEmbeddableURLResponse, dashboard, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncDashboards:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @parametrize
    async def test_method_get_embeddable_url(self, async_client: AsyncMetronome) -> None:
        dashboard = await async_client.v1.dashboards.get_embeddable_url(
            customer_id="4db51251-61de-4bfe-b9ce-495e244f3491",
            dashboard="invoices",
        )
        assert_matches_type(DashboardGetEmbeddableURLResponse, dashboard, path=["response"])

    @parametrize
    async def test_method_get_embeddable_url_with_all_params(self, async_client: AsyncMetronome) -> None:
        dashboard = await async_client.v1.dashboards.get_embeddable_url(
            customer_id="4db51251-61de-4bfe-b9ce-495e244f3491",
            dashboard="invoices",
            bm_group_key_overrides=[
                {
                    "group_key_name": "tenant_id",
                    "display_name": "Org ID",
                    "value_display_names": {
                        "48ecb18f358f": "bar",
                        "e358f3ce242d": "bar",
                    },
                }
            ],
            color_overrides=[
                {
                    "name": "Gray_dark",
                    "value": "#ff0000",
                }
            ],
            dashboard_options=[
                {
                    "key": "show_zero_usage_line_items",
                    "value": "false",
                },
                {
                    "key": "hide_voided_invoices",
                    "value": "true",
                },
            ],
        )
        assert_matches_type(DashboardGetEmbeddableURLResponse, dashboard, path=["response"])

    @parametrize
    async def test_raw_response_get_embeddable_url(self, async_client: AsyncMetronome) -> None:
        response = await async_client.v1.dashboards.with_raw_response.get_embeddable_url(
            customer_id="4db51251-61de-4bfe-b9ce-495e244f3491",
            dashboard="invoices",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        dashboard = await response.parse()
        assert_matches_type(DashboardGetEmbeddableURLResponse, dashboard, path=["response"])

    @parametrize
    async def test_streaming_response_get_embeddable_url(self, async_client: AsyncMetronome) -> None:
        async with async_client.v1.dashboards.with_streaming_response.get_embeddable_url(
            customer_id="4db51251-61de-4bfe-b9ce-495e244f3491",
            dashboard="invoices",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            dashboard = await response.parse()
            assert_matches_type(DashboardGetEmbeddableURLResponse, dashboard, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Iterable, Optional
from datetime import datetime
from typing_extensions import Literal, Required, Annotated, TypedDict

from ..._types import SequenceNotStr
from ..._utils import PropertyInfo
from ..shared_params.commit_specifier_input import CommitSpecifierInput
from ..shared_params.commit_hierarchy_configuration import CommitHierarchyConfiguration

__all__ = [
    "ContractEditCommitParams",
    "AccessSchedule",
    "AccessScheduleAddScheduleItem",
    "AccessScheduleRemoveScheduleItem",
    "AccessScheduleUpdateScheduleItem",
    "InvoiceSchedule",
    "InvoiceScheduleAddScheduleItem",
    "InvoiceScheduleRemoveScheduleItem",
    "InvoiceScheduleUpdateScheduleItem",
]


class ContractEditCommitParams(TypedDict, total=False):
    commit_id: Required[str]
    """ID of the commit to edit"""

    customer_id: Required[str]
    """ID of the customer whose commit is being edited"""

    access_schedule: AccessSchedule

    applicable_product_ids: Optional[SequenceNotStr[str]]
    """Which products the commit applies to.

    If applicable_product_ids, applicable_product_tags or specifiers are not
    provided, the commit applies to all products.
    """

    applicable_product_tags: Optional[SequenceNotStr[str]]
    """Which tags the commit applies to.

    If applicable_product_ids, applicable_product_tags or specifiers are not
    provided, the commit applies to all products.
    """

    description: str
    """Updated description for the commit"""

    hierarchy_configuration: CommitHierarchyConfiguration
    """Optional configuration for commit hierarchy access control"""

    invoice_contract_id: str
    """ID of contract to use for invoicing"""

    invoice_schedule: InvoiceSchedule

    name: str
    """Updated name for the commit"""

    priority: Optional[float]
    """
    If multiple commits are applicable, the one with the lower priority will apply
    first.
    """

    product_id: str

    rate_type: Literal["LIST_RATE", "COMMIT_RATE"]
    """
    If provided, updates the commit to use the specified rate type for current and
    future invoices. Previously finalized invoices will need to be voided and
    regenerated to reflect the rate type change.
    """

    specifiers: Optional[Iterable[CommitSpecifierInput]]
    """
    List of filters that determine what kind of customer usage draws down a commit
    or credit. A customer's usage needs to meet the condition of at least one of the
    specifiers to contribute to a commit's or credit's drawdown. This field cannot
    be used together with `applicable_product_ids` or `applicable_product_tags`.
    Instead, to target usage by product or product tag, pass those values in the
    body of `specifiers`.
    """


class AccessScheduleAddScheduleItem(TypedDict, total=False):
    amount: Required[float]

    ending_before: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]

    starting_at: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]


class AccessScheduleRemoveScheduleItem(TypedDict, total=False):
    id: Required[str]


class AccessScheduleUpdateScheduleItem(TypedDict, total=False):
    id: Required[str]

    amount: float

    ending_before: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]

    starting_at: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]


class AccessSchedule(TypedDict, total=False):
    add_schedule_items: Iterable[AccessScheduleAddScheduleItem]

    remove_schedule_items: Iterable[AccessScheduleRemoveScheduleItem]

    update_schedule_items: Iterable[AccessScheduleUpdateScheduleItem]


class InvoiceScheduleAddScheduleItem(TypedDict, total=False):
    timestamp: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]

    amount: float

    quantity: float

    unit_price: float


class InvoiceScheduleRemoveScheduleItem(TypedDict, total=False):
    id: Required[str]


class InvoiceScheduleUpdateScheduleItem(TypedDict, total=False):
    id: Required[str]

    amount: float

    quantity: float

    timestamp: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]

    unit_price: float


class InvoiceSchedule(TypedDict, total=False):
    add_schedule_items: Iterable[InvoiceScheduleAddScheduleItem]

    remove_schedule_items: Iterable[InvoiceScheduleRemoveScheduleItem]

    update_schedule_items: Iterable[InvoiceScheduleUpdateScheduleItem]
